from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from budget import common_text as text
from budget import models


class TransactionSerializer(serializers.ModelSerializer):
    owner = serializers.CharField(read_only=True)

    class Meta:
        model = models.Transaction
        fields = '__all__'

    def create(self, validated_data):
        request = self.context.get('request')
        validated_data['owner'] = request.user
        return super().create(validated_data)

    def validate(self, attrs):
        # validate the date with budget start and end date
        date = attrs.get('date')
        budget = attrs.get('line_item').budget
        if date < budget.start_date or date > budget.end_date:
            raise ValidationError(text.template_date_out_of_range.format(date, budget.start_date, budget.end_date))
        return attrs



# Generated by Django 2.2.17 on 2020-11-21 09:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Budget',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('start_date', models.DateField()),
                ('end_date', models.DateField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='LineItem',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('amount', models.FloatField()),
                ('type', models.CharField(choices=[('income', 'income'), ('expense', 'expense')], max_length=7)),
                ('category', models.CharField(max_length=100)),
                ('sharing', models.CharField(choices=[('personal', 'personal'), ('shared', 'shared')], max_length=8)),
                ('budget', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='budget.Budget')),
                ('owner', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('amount', models.FloatField()),
                ('type', models.CharField(choices=[('income', 'income'), ('expense', 'expense')], max_length=7)),
                ('merchant', models.CharField(max_length=100)),
                ('comment', models.TextField(blank=True)),
                ('date', models.DateField()),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('line_item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='budget.LineItem')),
                ('owner', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django_filters import rest_framework as django_filters

from budget import models


class TransactionFilter(django_filters.FilterSet):
    owner = django_filters.CharFilter(field_name='owner__username', lookup_expr='exact')

    class Meta:
        model = models.Transaction
        fields = {
            'amount': ['exact', 'lt', 'lte', 'gt', 'gte'],
            'type': ['exact'],
            'line_item': ['exact'],
            'merchant': ['exact'],
            'comment': ['contains'],
            'date': ['exact', 'lt', 'lte', 'gt', 'gte'],
            'last_modified': ['lt', 'lte', 'gt', 'gte']
        }



from django.urls import path

from budget import views

urlpatterns = [
    path('budget', views.BudgetListCreateView.as_view(), name='budget-lc'),
    path('budget/<uuid:id>', views.BudgetRetrieveUpdateDestroyView.as_view(), name='budget-rud'),
    path('lineitem', views.LineItemListCreateView.as_view(), name='line-item-lc'),
    path('lineitem/<uuid:id>', views.LineItemRetrieveUpdateDestroyView.as_view(), name='line-item-rud'),
    path('transaction', views.TransactionListCreateView.as_view(), name='transaction-lc'),
    path('transaction/<uuid:id>', views.TransactionRetrieveUpdateDestroyView.as_view(), name='transaction-rud')
]



from rest_framework import generics
from rest_framework.permissions import IsAuthenticated

from budget import serializers, models, filters


class BudgetListCreateView(generics.ListCreateAPIView):
    lookup_field = 'id'
    serializer_class = serializers.BudgetSerializer
    queryset = models.Budget.objects.all()
    permission_classes = (IsAuthenticated, )
    filterset_class = filters.BudgetFilter


class BudgetRetrieveUpdateDestroyView(generics.RetrieveUpdateDestroyAPIView):
    lookup_field = 'id'
    serializer_class = serializers.BudgetSerializer
    queryset = models.Budget.objects.all()
    permission_classes = (IsAuthenticated, )




import uuid

from django.contrib.auth import get_user_model
from django.db import models


class IdMixin(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta:
        abstract = True


class MoneyMixin(models.Model):
    amount = models.FloatField()
    type = models.CharField(max_length=7, choices=(('income', 'income'), ('expense', 'expense')))

    class Meta:
        abstract = True


class Budget(IdMixin):
    start_date = models.DateField()
    end_date = models.DateField()


class LineItem(IdMixin, MoneyMixin):
    budget = models.ForeignKey(Budget, null=True, on_delete=models.SET_NULL)
    category = models.CharField(max_length=100)
    sharing = models.CharField(max_length=8, choices=(('personal', 'personal'), ('shared', 'shared')))
    owner = models.ForeignKey(get_user_model(), editable=False, null=True, on_delete=models.SET_NULL)


class Transaction(IdMixin, MoneyMixin):
    line_item = models.ForeignKey(LineItem, null=True, on_delete=models.SET_NULL)
    owner = models.ForeignKey(get_user_model(), editable=False, null=True, on_delete=models.SET_NULL)
    merchant = models.CharField(max_length=100)
    comment = models.TextField(blank=True)
    date = models.DateField()
    last_modified = models.DateTimeField(auto_now=True)



from rest_framework import generics
from rest_framework.permissions import IsAuthenticated

from budget import serializers, models, filters


class TransactionListCreateView(generics.ListCreateAPIView):
    lookup_field = 'id'
    serializer_class = serializers.TransactionSerializer
    queryset = models.Transaction.objects.all()
    permission_classes = (IsAuthenticated, )
    filterset_class = filters.TransactionFilter


class TransactionRetrieveUpdateDestroyView(generics.RetrieveUpdateDestroyAPIView):
    lookup_field = 'id'
    serializer_class = serializers.TransactionSerializer
    queryset = models.Transaction.objects.all()
    permission_classes = (IsAuthenticated, )



from django_filters import rest_framework as django_filters

from budget import models


class BudgetFilter(django_filters.FilterSet):
    class Meta:
        model = models.Budget
        fields = {
            'start_date': ['exact', 'lt', 'lte', 'gt', 'gte'],
            'end_date': ['exact', 'lt', 'lte', 'gt', 'gte']
        }



from os import path
from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup_requires = [
    'setuptools_scm'
]

install_requires = [
    'django~=2.2',
    'djangorestframework~=3.11',
    'django-filter~=2.3'
]

setup(
    name='django-budget-backend',
    description='An API to manage your monthly budgets.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/personal-server/django-budget-backend',
    author='Sheldon Woodward',
    author_email='me@sheldonw.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django :: 2.2',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Session',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='django budget backend',
    packages=['budget'],
    python_requires='~=3.8',
    setup_requires=setup_requires,
    install_requires=install_requires,
    use_scm_version=True
)



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from budget import common_text as text
from budget import models


class LineItemSerializer(serializers.ModelSerializer):
    owner = serializers.CharField(read_only=True)

    class Meta:
        model = models.LineItem
        fields = '__all__'

    def create(self, validated_data):
        if validated_data['sharing'] == 'personal':
            request = self.context.get('request')
            validated_data['owner'] = request.user
        return super().create(validated_data)

    def validate_sharing(self, value):
        if self.instance is not None and self.instance.sharing != value:
            raise ValidationError(text.template_cannot_edit_property.format('sharing'))
        return value
